@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,20 +21,20 @@ Means to create XML elements from Nuitka tree nodes and to convert the
 XML tree to ASCII or output it.
 """
 
-from nuitka import Tracing, Utils
+from . import Tracing, Utils
 
 try:
     import lxml.etree
 except ImportError:
     lxml = None
 
-def toString( xml ):
-    return lxml.etree.tostring( xml, pretty_print = True )
+def toString(xml):
+    return lxml.etree.tostring(xml, pretty_print = True)
 
-def dump( xml  ):
-    value = toString( xml ).rstrip()
+def dump(xml):
+    value = toString(xml).rstrip()
 
     if Utils.python_version >= 300:
-        value = value.decode( "utf-8" )
+        value = value.decode("utf-8")
 
-    Tracing.printLine( value )
+    Tracing.printLine(value)
