@@ -48,10 +48,9 @@ def getComparisonExpressionCode(to_name, comparator, left_name, right_name,
         )
 
         getErrorExitCode(
-            check_name      = to_name,
-            quick_exception = None,
-            emit            = emit,
-            context         = context
+            check_name  = to_name,
+            emit        = emit,
+            context     = context
         )
 
         getReleaseCode(
@@ -168,10 +167,9 @@ def getComparisonExpressionBoolCode(comparator, left_name, right_name, emit,
         )
 
         getErrorExitBoolCode(
-            condition       = "%s == -1" % operator_res_name,
-            quick_exception = None,
-            emit            = emit,
-            context         = context
+            condition = "%s == -1" % operator_res_name,
+            emit      = emit,
+            context   = context
         )
 
         condition = "%s == %d" % (
@@ -239,10 +237,9 @@ def getComparisonExpressionBoolCode(comparator, left_name, right_name, emit,
         )
 
         getErrorExitBoolCode(
-            condition       = "%s == -1" % operator_res_name,
-            quick_exception = None,
-            emit            = emit,
-            context         = context
+            condition = "%s == -1" % operator_res_name,
+            emit      = emit,
+            context   = context
         )
 
         condition = "%s == 1" % (
@@ -321,10 +318,9 @@ def getBuiltinIsinstanceBoolCode(inst_name, cls_name, emit, context):
     )
 
     getErrorExitBoolCode(
-        condition       = "%s == -1" % res_name,
-        quick_exception = None,
-        emit            = emit,
-        context         = context
+        condition = "%s == -1" % res_name,
+        emit      = emit,
+        context   = context
     )
 
     getBranchingCode("%s == 1" % res_name, emit, context)
