@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,24 +22,19 @@ Language independent, the amount of the spaces is not configurable, as it needs
 to be the same as in templates.
 """
 
-def _indentedCode( codes, count ):
+def _indentedCode(codes, count):
     return "\n".join(
-        " " * count + line if (line and not line.startswith( "#" )) else line for line in codes
+        " " * count + line
+          if (line and not line.startswith( "#" )) else
+        line for line in codes
     )
 
-def indented( codes, level = 1, vert_block = False ):
-    if type( codes ) is str:
-        codes = codes.split( "\n" )
+def indented(codes, level = 1, vert_block = False):
+    if type(codes) is str:
+        codes = codes.split("\n")
 
     if vert_block and codes != [ "" ]:
+        codes.insert(0, "")
+        codes.append("")
 
-        codes.insert( 0, "" )
-        codes.append( "" )
-
-    return _indentedCode( codes, level * 4 )
-
-def getBlockCode( codes ):
-    if type( codes ) is str:
-        assert codes == codes.rstrip(), codes
-
-    return "{\n%s\n}" % indented( codes )
+    return _indentedCode(codes, level * 4)
