@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,46 +25,44 @@ to variables only ever read.
 """
 
 
-from .NodeBases import NodeBase, CompileTimeConstantExpressionMixin
-
-from .ConstantRefNodes import ExpressionConstantRef
-
-from nuitka.optimizations import BuiltinOptimization
-
 from nuitka.Builtins import (
+    builtin_anon_names,
     builtin_exception_names,
     builtin_exception_values,
-    builtin_anon_names,
     builtin_names
 )
-
+from nuitka.optimizations import BuiltinOptimization
 from nuitka.Utils import python_version
 
-class ExpressionBuiltinRefBase( CompileTimeConstantExpressionMixin, NodeBase ):
-    def __init__( self, builtin_name, source_ref ):
+from .ConstantRefNodes import ExpressionConstantRef
+from .NodeBases import CompileTimeConstantExpressionMixin, NodeBase
+
+
+class ExpressionBuiltinRefBase(CompileTimeConstantExpressionMixin, NodeBase):
+    def __init__(self, builtin_name, source_ref):
         NodeBase.__init__( self, source_ref = source_ref )
         CompileTimeConstantExpressionMixin.__init__( self )
 
         self.builtin_name = builtin_name
 
-    def makeCloneAt( self, source_ref ):
-        return self.__class__( self.builtin_name, source_ref )
+    def makeCloneAt(self, source_ref):
+        return self.__class__(self.builtin_name, source_ref)
 
-    def getDetails( self ):
+    def getDetails(self):
         return { "builtin_name" : self.builtin_name }
 
-    def getBuiltinName( self ):
+    def getBuiltinName(self):
         return self.builtin_name
 
-    def mayHaveSideEffects( self ):
+    def mayHaveSideEffects(self):
         # Referencing the builtin name has no side effect
         return False
 
 
-class ExpressionBuiltinRef( ExpressionBuiltinRefBase ):
+class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_REF"
 
-    def __init__( self, builtin_name, source_ref ):
+    def __init__(self, builtin_name, source_ref):
         assert builtin_name in builtin_names, builtin_name
 
         ExpressionBuiltinRefBase.__init__(
@@ -73,14 +71,14 @@ class ExpressionBuiltinRef( ExpressionBuiltinRefBase ):
             source_ref   = source_ref
         )
 
-    def isCompileTimeConstant( self ):
+    def isCompileTimeConstant(self):
         # Virtual method, pylint: disable=R0201
         return True
 
-    def getCompileTimeConstant( self ):
+    def getCompileTimeConstant(self):
         return __builtins__[ self.builtin_name ]
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         quick_names = {
             "None"      : None,
             "True"      : True,
@@ -91,55 +89,63 @@ class ExpressionBuiltinRef( ExpressionBuiltinRefBase ):
 
         if self.builtin_name in quick_names:
             new_node = ExpressionConstantRef(
-                constant   = quick_names[ self.builtin_name ],
+                constant   = quick_names[self.builtin_name],
                 source_ref = self.getSourceReference()
             )
 
-            return new_node, "new_constant", "Builtin constant %s resolved" % self.builtin_name
+            return new_node, "new_constant", """\
+Builtin constant %s resolved""" % self.builtin_name
 
         return self, None, None
 
-    def computeExpressionCall( self, call_node, constraint_collection ):
+    def computeExpressionCall(self, call_node, constraint_collection):
         from nuitka.optimizations.OptimizeBuiltinCalls import computeBuiltinCall
 
-        return computeBuiltinCall(
+        new_node, tags, message = computeBuiltinCall(
             call_node = call_node,
             called    = self
         )
 
-    def getStringValue( self ):
+        if new_node.isExpressionBuiltinLocals() or \
+           new_node.isExpressionBuiltinEval():
+            constraint_collection.assumeUnclearLocals(self.source_ref)
+
+        return new_node, tags, message
+
+    def getStringValue(self):
         return repr( self.getCompileTimeConstant() )
 
-    def isKnownToBeIterable( self, count ):
+    def isKnownToBeIterable(self, count):
         # TODO: Why yes, some may be, could be told here.
         return None
 
-    def mayProvideReference( self ):
-        # Dedicated code returns which returns from builtin module dictionary, but isn't
-        # available for Python3 yet.
+    def mayProvideReference(self):
+        # Dedicated code returns which returns from builtin module dictionary,
+        # but isn't available for Python3 yet.
 
         return python_version >= 300
 
 
-class ExpressionBuiltinOriginalRef( ExpressionBuiltinRef ):
+
+class ExpressionBuiltinOriginalRef(ExpressionBuiltinRef):
     kind = "EXPRESSION_BUILTIN_ORIGINAL_REF"
 
-    def isCompileTimeConstant( self ):
+    def isCompileTimeConstant(self):
         # TODO: Actually the base class should not be constant and this one should be.
 
         # Virtual method, pylint: disable=R0201
         return False
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
 
         # Needs whole program analysis, we don't really know much about it.
         return self, None, None
 
 
-class ExpressionBuiltinAnonymousRef( ExpressionBuiltinRefBase ):
+class ExpressionBuiltinAnonymousRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_ANONYMOUS_REF"
 
-    def __init__( self, builtin_name, source_ref ):
+    def __init__(self, builtin_name, source_ref):
         assert builtin_name in builtin_anon_names
 
         ExpressionBuiltinRefBase.__init__(
@@ -148,30 +154,30 @@ class ExpressionBuiltinAnonymousRef( ExpressionBuiltinRefBase ):
             source_ref   = source_ref
         )
 
-    def isCompileTimeConstant( self ):
+    def isCompileTimeConstant(self):
         # Virtual method, pylint: disable=R0201
         return True
 
-    def mayProvideReference( self ):
-        # No reference provided from this, there are just a global identifiers, or
-        # accesses to them.
+    def mayProvideReference(self):
+        # No reference provided from this, there are just a global identifiers,
+        # or accesses to them.
 
         return False
 
-    def getCompileTimeConstant( self ):
+    def getCompileTimeConstant(self):
         return builtin_anon_names[ self.builtin_name ]
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         return self, None, None
 
-    def getStringValue( self ):
+    def getStringValue(self):
         return repr( self.getCompileTimeConstant() )
 
 
-class ExpressionBuiltinExceptionRef( ExpressionBuiltinRefBase ):
+class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
     kind = "EXPRESSION_BUILTIN_EXCEPTION_REF"
 
-    def __init__( self, exception_name, source_ref ):
+    def __init__(self, exception_name, source_ref):
         assert exception_name in builtin_exception_names
 
         ExpressionBuiltinRefBase.__init__(
@@ -180,40 +186,44 @@ class ExpressionBuiltinExceptionRef( ExpressionBuiltinRefBase ):
             source_ref   = source_ref
         )
 
-    def getDetails( self ):
+    def getDetails(self):
         return { "exception_name" : self.builtin_name }
 
     getExceptionName = ExpressionBuiltinRefBase.getBuiltinName
 
-    def isCompileTimeConstant( self ):
+    def isCompileTimeConstant(self):
         # Virtual method, pylint: disable=R0201
         return True
 
-    def mayProvideReference( self ):
+    def mayRaiseException(self, exception_type):
+        return False
+
+    def mayProvideReference(self):
         # No reference provided from this, it's just a global identifier.
 
         return False
 
-    def getCompileTimeConstant( self ):
+    def getCompileTimeConstant(self):
         return builtin_exception_values[ self.builtin_name ]
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         # Children can tell all we need to know, pylint: disable=W0613
 
         return self, None, None
 
-    def computeExpressionCall( self, call_node, constraint_collection ):
+    def computeExpressionCall(self, call_node, constraint_collection):
         exception_name = self.getExceptionName()
 
-        # TODO: Keyword only arguments of it, are not properly handled yet.
+        # TODO: Keyword only arguments of it, are not properly handled yet by
+        # the built-in call code.
         if exception_name == "ImportError" and python_version >= 330:
             kw = call_node.getCallKw()
 
-            if not kw.isExpressionConstantRef() and kw.getConstant() == {}:
+            if not kw.isExpressionConstantRef() or kw.getConstant() != {}:
                 return call_node, None, None
 
-        def createBuiltinMakeException( args, source_ref ):
-            from nuitka.nodes.ExceptionNodes import ExpressionBuiltinMakeException
+        def createBuiltinMakeException(args, source_ref):
+            from .ExceptionNodes import ExpressionBuiltinMakeException
 
             return ExpressionBuiltinMakeException(
                 exception_name = exception_name,
