@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,39 +15,43 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Reformulation of function statements.
 
-from nuitka import Utils, SyntaxErrors
+Consult the developer manual for information. TODO: Add ability to sync
+source code comments with developer manual sections.
 
-from nuitka.nodes.ParameterSpecs import ParameterSpec
+"""
 
-from nuitka.nodes.VariableRefNodes import ExpressionTargetVariableRef
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka import SyntaxErrors, Utils
+from nuitka.nodes.AssignNodes import StatementAssignmentVariable
 from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCreation,
     ExpressionFunctionBody,
+    ExpressionFunctionCreation,
     ExpressionFunctionRef
 )
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
+from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
+from nuitka.nodes.VariableRefNodes import ExpressionTargetVariableRef
 
 from .Helpers import (
-    makeStatementsSequenceFromStatement,
-    makeDictCreationOrConstant,
+    buildNode,
+    buildNodeList,
     buildStatementsNode,
     extractDocFromBody,
-    buildNodeList,
-    buildNode,
-    getKind
+    getKind,
+    makeDictCreationOrConstant,
+    makeStatementsSequenceFromStatement,
+    popIndicatorVariable,
+    pushIndicatorVariable
 )
 
-def buildFunctionNode( provider, node, source_ref ):
-    assert getKind( node ) == "FunctionDef"
 
-    # Remove "exec" flag if any.
-    source_ref = source_ref.getExecReference( False )
+def buildFunctionNode(provider, node, source_ref):
+    assert getKind( node ) == "FunctionDef"
 
     function_statements, function_doc = extractDocFromBody( node )
 
@@ -64,7 +68,7 @@ def buildFunctionNode( provider, node, source_ref ):
 
     decorators = buildNodeList(
         provider   = provider,
-        nodes      = reversed( node.decorator_list ),
+        nodes      = reversed(node.decorator_list),
         source_ref = source_ref
     )
 
@@ -78,6 +82,8 @@ def buildFunctionNode( provider, node, source_ref ):
         provider, node, function_body, source_ref
     )
 
+    pushIndicatorVariable(Ellipsis)
+
     function_statements_body = buildStatementsNode(
         provider   = function_body,
         nodes      = function_statements,
@@ -85,6 +91,8 @@ def buildFunctionNode( provider, node, source_ref ):
         source_ref = source_ref
     )
 
+    popIndicatorVariable()
+
     if function_body.isExpressionFunctionBody() and function_body.isGenerator():
         # TODO: raise generator exit?
         pass
@@ -112,9 +120,11 @@ def buildFunctionNode( provider, node, source_ref ):
             )
         )
 
-    function_body.setBody( function_statements_body )
+    function_body.setBody(
+        function_statements_body
+    )
 
-    annotations = buildParameterAnnotations( provider, node, source_ref )
+    annotations = buildParameterAnnotations(provider, node, source_ref)
 
     decorated_body = ExpressionFunctionCreation(
         function_ref = ExpressionFunctionRef(
@@ -160,7 +170,8 @@ def buildFunctionNode( provider, node, source_ref ):
         source_ref   = source_ref
     )
 
-def buildParameterKwDefaults( provider, node, function_body, source_ref ):
+
+def buildParameterKwDefaults(provider, node, function_body, source_ref):
     # Build keyword only arguments default values. We are hiding here, that it
     # is a Python3 only feature.
 
@@ -197,7 +208,7 @@ def buildParameterKwDefaults( provider, node, function_body, source_ref ):
 
     return kw_defaults
 
-def buildParameterAnnotations( provider, node, source_ref ):
+def buildParameterAnnotations(provider, node, source_ref):
     # Too many branches, because there is too many cases, pylint: disable=R0912
 
     # Build annotations. We are hiding here, that it is a Python3 only feature.
@@ -207,7 +218,7 @@ def buildParameterAnnotations( provider, node, source_ref ):
     keys = []
     values = []
 
-    def addAnnotation( key, value ):
+    def addAnnotation(key, value):
         keys.append(
             ExpressionConstantRef(
                 constant      = key,
@@ -217,7 +228,7 @@ def buildParameterAnnotations( provider, node, source_ref ):
         )
         values.append( value )
 
-    def extractArg( arg ):
+    def extractArg(arg):
         if getKind( arg ) == "Name":
             assert arg.annotation is None
         elif getKind( arg ) == "arg":
@@ -233,10 +244,10 @@ def buildParameterAnnotations( provider, node, source_ref ):
             assert False, getKind( arg )
 
     for arg in node.args.args:
-        extractArg( arg )
+        extractArg(arg)
 
     for arg in node.args.kwonlyargs:
-        extractArg( arg )
+        extractArg(arg)
 
     if Utils.python_version < 340:
         if node.args.varargannotation is not None:
@@ -277,12 +288,12 @@ def buildParameterAnnotations( provider, node, source_ref ):
     else:
         return None
 
-def buildParameterSpec( name, node, source_ref ):
+def buildParameterSpec(name, node, source_ref):
     kind = getKind( node )
 
     assert kind in ( "FunctionDef", "Lambda" ), "unsupported for kind " + kind
 
-    def extractArg( arg ):
+    def extractArg(arg):
         if type( arg ) is str or arg is None:
             return arg
         elif getKind( arg ) == "Name":
@@ -302,7 +313,7 @@ def buildParameterSpec( name, node, source_ref ):
                          [],
         list_star_arg  = extractArg( node.args.vararg ),
         dict_star_arg  = extractArg( node.args.kwarg ),
-        default_count  = len( node.args.defaults )
+        default_count  = len(node.args.defaults)
     )
 
     message = result.checkValid()
