@@ -16,122 +16,124 @@
 #     limitations under the License.
 #
 
-
+from __future__ import print_function
 
 def simple_comparisons(x, y):
     if 'a' <= x <= y <= 'z':
-        print "One"
+        print("One")
 
     if 'a' <= x <= 'z':
-        print "Two"
+        print("Two")
 
     if 'a' <= x > 'z':
-        print "Three"
+        print("Three")
 
-print "Simple comparisons:"
+print("Simple comparisons:")
 
-simple_comparisons( "c", "d" )
+simple_comparisons("c", "d")
 
 def side_effect():
-    print "<side_effect>"
+    print("<side_effect>")
 
     return 7
 
 def side_effect_comparisons():
-    print "Should have side effect:"
+    print("Should have side effect:")
+    print(1 < side_effect() < 9)
 
-    print 1 < side_effect() < 9
+    print("Should not have side effect due to short circuit:")
+    print(3 < 2 < side_effect() < 9)
 
-    print "Should not have side effect due to short circuit"
-
-    print 3 < 2 < side_effect() < 9
-
-print "Check for expected side effects only:"
+print("Check for expected side effects only:")
 
 side_effect_comparisons()
 
 def function_torture_is():
-    a = ( 1, 2, 3 )
+    a = (1, 2, 3)
 
     for x in a:
         for y in a:
             for z in a:
-                print x, y, z, ":", x is y is z, x is not y is not z
+                print(x, y, z, ":", x is y is z, x is not y is not z)
 
 function_torture_is()
 
-print "Check if lambda can have expression chains:",
+print("Check if lambda can have expression chains:", end="")
 
 def function_lambda_with_chain():
 
-    a = ( 1, 2, 3 )
+    a = (1, 2, 3)
 
     x = lambda x : x[0] < x[1] < x[2]
 
-    print x(a)
+    print("lambda result is", x(a))
 
 function_lambda_with_chain()
 
-print "Check if generators can have expression chains:",
+print("Check if generators can have expression chains:", end = "")
 
 def generator_function_with_chain():
-    x = ( 1, 2, 3 )
+    x = (1, 2, 3)
 
     yield x[0] < x[1] < x[2]
 
-print list( generator_function_with_chain() )
+print(list(generator_function_with_chain()))
 
-print "Check if list contractions can have expression chains:",
+print("Check if list contractions can have expression chains:", end = "")
 
 def contraction_with_chain():
     return [ x[0] < x[1] < x[2] for x in [ ( 1, 2, 3 ) ] ]
 
-print contraction_with_chain()
+print(contraction_with_chain())
+
+print("Check if generator expressions can have expression chains:", end = "")
 
 def genexpr_with_chain():
     return ( x[0] < x[1] < x[2] for x in [ ( 1, 2, 3 ) ] )
 
-print list( genexpr_with_chain() )
+print(list(genexpr_with_chain()))
+
+print("Check if class bodies can have expression chains:", end = "")
 
 class class_with_chain:
     x = ( 1, 2, 3 )
-    print x[0] < x[1] < x[2]
+    print(x[0] < x[1] < x[2])
 
 x = ( 1, 2, 3 )
-print x[0] < x[1] < x[2]
+print(x[0] < x[1] < x[2])
 
 class CustomOps(int):
     def __lt__(self, other):
-        print "<", self, other
+        print("enter <", self, other)
 
         return True
 
     def __gt__(self, other):
-        print ">", self, other
+        print("enter >", self, other)
 
         return False
 
 
-print "Custom ops, to enforce chain eval order and short circuit:"
-print CustomOps( 7 ) < CustomOps( 8 ) > CustomOps( 6 )
+print("Custom ops, to enforce chain eval order and short circuit:", end = "")
+print(CustomOps(7) < CustomOps(8) > CustomOps(6))
 
-print "Custom ops, do short circuit:"
-print CustomOps( 8 ) > CustomOps( 7 ) < CustomOps( 6 )
+print("Custom ops, doing short circuit:", end = "")
+print(CustomOps(8) > CustomOps(7) < CustomOps(6))
 
 def inOperatorChain():
-    print "In operator chains:"
-    print 3 in [3,4] in [[3,4]]
-    print 3 in [3,4] not in [[3,4]]
+    print("In operator chains:")
+    print(3 in [3,4] in [[3,4]])
+    print(3 in [3,4] not in [[3,4]])
 
     if 3 in [3,4] in [[3,4]]:
-       print "Yes"
+       print("Yes")
     else:
-       print "No"
+       print("No")
 
     if 3 in [3,4] not in [[3,4]]:
-       print "Yes"
+       print("Yes")
     else:
-       print "No"
+       print("No")
 
 
 inOperatorChain()
@@ -147,24 +149,24 @@ class A(object):
         return "<Value %s %d>" % ( self.name, self.value )
 
     def __lt__(self, other):
-        print "less than called for:", self, other, self.value, other.value, self.value < other.value
+        print("less than called for:", self, other, self.value, other.value, self.value < other.value)
 
         if self.value < other.value:
-            print "good"
+            print("good")
             return 7
         else:
-            print "bad"
+            print("bad")
             return 0
 
 a = A("a",1)
 b = A("b",2)
 c = A("c",0)
 
-print a < b < c
-print "*" * 80
+print(a < b < c)
+print("*" * 80)
 
 a = A("a",2)
 b = A("b",1)
 c = A("c",0)
 
-print a < b < c
+print(a < b < c)
