@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -19,7 +19,7 @@ print( "some_package.star_importing, doing the star import" )
 
 print( "Before", sorted( dir() ) )
 
-from posix import *
+from .pyexpat import *
 
 lala = 1
 print( "After", sorted( dir() ) )
